/* eslint-disable no-magic-numbers */
import { Feature } from 'ol';
import VectorLayer from 'ol/layer/Vector';
import VectorSource from 'ol/source/Vector';
import { useEffect, useMemo } from 'react';
import { usePointToProjection } from '@/utils/map/usePointToProjection';
import MapElement from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/elements/MapElement';
import { useSelector } from 'react-redux';
import {
  arrowTypesSelector,
  bioShapesSelector,
  lineTypesSelector,
} from '@/redux/shapes/shapes.selectors';
import { MapInstance } from '@/types/map';
import {
  HorizontalAlign,
  VerticalAlign,
} from '@/components/Map/MapViewer/MapViewerVector/MapViewerVector.types';
import { modelElementsSelector } from '@/redux/modelElements/modelElements.selector';
import { currentModelIdSelector } from '@/redux/models/models.selectors';
import { getModelElements } from '@/redux/modelElements/modelElements.thunks';
import { useAppDispatch } from '@/redux/hooks/useAppDispatch';
import CompartmentSquare from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/elements/CompartmentSquare';
import CompartmentCircle from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/elements/CompartmentCircle';
import { ModelElement } from '@/types/models';
import Glyph from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/elements/Glyph';
import CompartmentPathway from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/elements/CompartmentPathway';
import Reaction from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/reaction/Reaction';
import { newReactionsDataSelector } from '@/redux/newReactions/newReactions.selectors';
import { getNewReactions } from '@/redux/newReactions/newReactions.thunks';
import { VECTOR_MAP_LAYER_TYPE } from '@/components/Map/MapViewer/MapViewerVector/MapViewerVector.constants';

export const useOlMapReactionsLayer = ({
  mapInstance,
}: {
  mapInstance: MapInstance;
}): VectorLayer<VectorSource<Feature>> => {
  const dispatch = useAppDispatch();
  const modelElements = useSelector(modelElementsSelector);
  const modelReactions = useSelector(newReactionsDataSelector);
  const currentModelId = useSelector(currentModelIdSelector);
  useEffect(() => {
    dispatch(getModelElements(currentModelId));
    dispatch(getNewReactions(currentModelId));
  }, [currentModelId, dispatch]);

  const pointToProjection = usePointToProjection();
  const shapes = useSelector(bioShapesSelector);
  const lineTypes = useSelector(lineTypesSelector);
  const arrowTypes = useSelector(arrowTypesSelector);

  const reactions = useMemo(() => {
    return modelReactions.map(reaction => {
      const reactionShapes = shapes && shapes[reaction.sboTerm];
      if (!reactionShapes) {
        return [];
      }
      const reactionObject = new Reaction({
        id: reaction.id,
        line: reaction.line,
        products: reaction.products,
        reactants: reaction.reactants,
        modifiers: reaction.modifiers,
        operators: reaction.operators,
        zIndex: reaction.z,
        lineTypes,
        arrowTypes,
        shapes: reactionShapes,
        pointToProjection,
        mapInstance,
      });
      return reactionObject.features;
    });
  }, [arrowTypes, lineTypes, mapInstance, modelReactions, pointToProjection, shapes]);

  const elements: Array<
    MapElement | CompartmentCircle | CompartmentSquare | CompartmentPathway | Glyph
  > = useMemo(() => {
    if (!modelElements || !shapes) {
      return [];
    }

    const validElements: Array<
      MapElement | CompartmentCircle | CompartmentSquare | CompartmentPathway | Glyph
    > = [];
    modelElements.content.forEach((element: ModelElement) => {
      if (element.glyph) {
        const glyph = new Glyph({
          id: element.glyph.id,
          x: element.x,
          y: element.y,
          width: element.width,
          height: element.height,
          zIndex: element.z,
          pointToProjection,
          mapInstance,
        });
        validElements.push(glyph);
        return;
      }

      if (element.sboTerm === 'SBO:0000290') {
        const compartmentProps = {
          id: element.id,
          x: element.x,
          y: element.y,
          nameX: element.nameX,
          nameY: element.nameY,
          nameHeight: element.nameHeight,
          nameWidth: element.nameWidth,
          width: element.width,
          height: element.height,
          zIndex: element.z,
          innerWidth: element.innerWidth,
          outerWidth: element.outerWidth,
          thickness: element.thickness,
          fontColor: element.fontColor,
          fillColor: element.fillColor,
          borderColor: element.borderColor,
          nameVerticalAlign: element.nameVerticalAlign as VerticalAlign,
          nameHorizontalAlign: element.nameHorizontalAlign as HorizontalAlign,
          text: element.name,
          fontSize: element.fontSize,
          pointToProjection,
          mapInstance,
        };
        if (element.shape === 'OVAL_COMPARTMENT') {
          validElements.push(new CompartmentCircle(compartmentProps));
        } else if (element.shape === 'SQUARE_COMPARTMENT') {
          validElements.push(new CompartmentSquare(compartmentProps));
        } else if (element.shape === 'PATHWAY') {
          validElements.push(new CompartmentPathway(compartmentProps));
        }
        return;
      }
      const elementShapes = shapes[element.sboTerm];
      if (elementShapes) {
        validElements.push(
          new MapElement({
            id: element.id,
            shapes: elementShapes,
            x: element.x,
            y: element.y,
            nameX: element.nameX,
            nameY: element.nameY,
            nameHeight: element.nameHeight,
            nameWidth: element.nameWidth,
            width: element.width,
            height: element.height,
            zIndex: element.z,
            lineWidth: element.lineWidth,
            lineType: element.borderLineType,
            fontColor: element.fontColor,
            fillColor: element.fillColor,
            borderColor: element.borderColor,
            nameVerticalAlign: element.nameVerticalAlign as VerticalAlign,
            nameHorizontalAlign: element.nameHorizontalAlign as HorizontalAlign,
            homodimer: element.homodimer,
            activity: element.activity,
            text: element.name,
            fontSize: element.fontSize,
            pointToProjection,
            mapInstance,
            modifications: element.modificationResidues,
            lineTypes,
            bioShapes: shapes,
          }),
        );
      }
    });
    return validElements;
  }, [modelElements, shapes, pointToProjection, mapInstance, lineTypes]);

  const features = useMemo(() => {
    const reactionsFeatures = reactions.flat();
    const elementsFeatures = elements.map(element => element.feature);
    return [...reactionsFeatures, ...elementsFeatures];
  }, [elements, reactions]);

  const vectorSource = useMemo(() => {
    return new VectorSource({
      features,
    });
  }, [features]);

  return useMemo(() => {
    const vectorLayer = new VectorLayer({
      source: vectorSource,
    });
    vectorLayer.set('type', VECTOR_MAP_LAYER_TYPE);
    return vectorLayer;
  }, [vectorSource]);
};
